@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import contextlib
 import csv
 import itertools
 import json
@@ -12,21 +13,23 @@ from pathlib import Path
 from subprocess import CalledProcessError
 from typing import TYPE_CHECKING
 from typing import Any
-from typing import cast
 
 from cleo.io.null_io import NullIO
-from poetry.core.packages.file_dependency import FileDependency
 from poetry.core.packages.utils.link import Link
-from poetry.core.pyproject.toml import PyProjectTOML
 
 from poetry.installation.chef import Chef
+from poetry.installation.chef import ChefBuildError
 from poetry.installation.chooser import Chooser
 from poetry.installation.operations import Install
 from poetry.installation.operations import Uninstall
 from poetry.installation.operations import Update
+from poetry.installation.wheel_installer import WheelInstaller
+from poetry.puzzle.exceptions import SolverProblemError
 from poetry.utils._compat import decode
 from poetry.utils.authenticator import Authenticator
 from poetry.utils.env import EnvCommandError
+from poetry.utils.helpers import atomic_open
+from poetry.utils.helpers import get_file_hash
 from poetry.utils.helpers import pluralize
 from poetry.utils.helpers import remove_directory
 from poetry.utils.pip import pip_install
@@ -40,7 +43,7 @@ if TYPE_CHECKING:
 
     from poetry.config.config import Config
     from poetry.installation.operations.operation import Operation
-    from poetry.repositories import Pool
+    from poetry.repositories import RepositoryPool
     from poetry.utils.env import Env
 
 
@@ -48,7 +51,7 @@ class Executor:
     def __init__(
         self,
         env: Env,
-        pool: Pool,
+        pool: RepositoryPool,
         config: Config,
         io: IO,
         parallel: bool | None = None,
@@ -59,11 +62,10 @@ class Executor:
         self._dry_run = False
         self._enabled = True
         self._verbose = False
-        self._authenticator = Authenticator(
-            config, self._io, disable_cache=disable_cache
+        self._wheel_installer = WheelInstaller(self._env)
+        self._use_modern_installation = config.get(
+            "installer.modern-installation", True
         )
-        self._chef = Chef(config, self._env)
-        self._chooser = Chooser(pool, self._env, config)
 
         if parallel is None:
             parallel = config.get("installer.parallel", True)
@@ -75,6 +77,12 @@ class Executor:
         else:
             self._max_workers = 1
 
+        self._authenticator = Authenticator(
+            config, self._io, disable_cache=disable_cache, pool_size=self._max_workers
+        )
+        self._chef = Chef(config, self._env, pool)
+        self._chooser = Chooser(pool, self._env, config)
+
         self._executor = ThreadPoolExecutor(max_workers=self._max_workers)
         self._total_operations = 0
         self._executed_operations = 0
@@ -116,6 +124,9 @@ class Executor:
 
         return self
 
+    def enable_bytecode_compilation(self, enable: bool = True) -> None:
+        self._wheel_installer.enable_bytecode_compilation(enable)
+
     def pip_install(
         self, req: Path, upgrade: bool = False, editable: bool = False
     ) -> int:
@@ -141,10 +152,18 @@ class Executor:
         if operations and (self._enabled or self._dry_run):
             self._display_summary(operations)
 
-        # We group operations by priority
-        groups = itertools.groupby(operations, key=lambda o: -o.priority)
         self._sections = {}
         self._yanked_warnings = []
+
+        # pip has to be installed first without parallelism if we install via pip
+        for i, op in enumerate(operations):
+            if op.package.name == "pip":
+                wait([self._executor.submit(self._execute_operation, op)])
+                del operations[i]
+                break
+
+        # We group operations by priority
+        groups = itertools.groupby(operations, key=lambda o: -o.priority)
         for _, group in groups:
             tasks = []
             serial_operations = []
@@ -262,7 +281,7 @@ class Executor:
             # error to be picked up by the error handler.
             if result == -2:
                 raise KeyboardInterrupt
-        except Exception as e:
+        except Exception as e:  # noqa: PIE786
             try:
                 from cleo.ui.exception_trace import ExceptionTrace
 
@@ -281,6 +300,35 @@ class Executor:
                 with self._lock:
                     trace = ExceptionTrace(e)
                     trace.render(io)
+                    if isinstance(e, ChefBuildError):
+                        pkg = operation.package
+                        pip_command = "pip wheel --use-pep517"
+                        if pkg.develop:
+                            requirement = pkg.source_url
+                            pip_command += " --editable"
+                        else:
+                            requirement = (
+                                pkg.to_dependency().to_pep_508().split(";")[0].strip()
+                            )
+                        io.write_line("")
+                        io.write_line(
+                            "<info>"
+                            "Note: This error originates from the build backend,"
+                            " and is likely not a problem with poetry"
+                            f" but with {pkg.pretty_name} ({pkg.full_pretty_version})"
+                            " not supporting PEP 517 builds. You can verify this by"
+                            f" running '{pip_command} \"{requirement}\"'."
+                            "</info>"
+                        )
+                    elif isinstance(e, SolverProblemError):
+                        pkg = operation.package
+                        io.write_line("")
+                        io.write_line(
+                            "<error>"
+                            "Cannot resolve build-system.requires"
+                            f" for {pkg.pretty_name}."
+                            "</error>"
+                        )
                     io.write_line("")
             finally:
                 with self._lock:
@@ -308,10 +356,12 @@ class Executor:
             if self.supports_fancy_output():
                 self._write(
                     operation,
-                    f"  <fg=default;options=bold,dark>•</> {operation_message}: "
-                    "<fg=default;options=bold,dark>Skipped</> "
-                    "<fg=default;options=dark>for the following reason:</> "
-                    f"<fg=default;options=bold,dark>{operation.skip_reason}</>",
+                    (
+                        f"  <fg=default;options=bold,dark>•</> {operation_message}: "
+                        "<fg=default;options=bold,dark>Skipped</> "
+                        "<fg=default;options=dark>for the following reason:</> "
+                        f"<fg=default;options=bold,dark>{operation.skip_reason}</>"
+                    ),
                 )
 
             self._skipped[operation.job_type] += 1
@@ -319,8 +369,6 @@ class Executor:
             return 0
 
         if not self._enabled or self._dry_run:
-            self._io.write_line(f"  <fg=blue;options=bold>•</> {operation_message}")
-
             return 0
 
         result: int = getattr(self, f"_execute_{method}")(operation)
@@ -461,18 +509,22 @@ class Executor:
         message = f"  <fg=blue;options=bold>•</> {op_msg}: <info>Removing...</info>"
         self._write(operation, message)
 
-        return self._remove(operation)
+        return self._remove(operation.package)
 
     def _install(self, operation: Install | Update) -> int:
         package = operation.package
-        if package.source_type == "directory":
-            return self._install_directory(operation)
+        if package.source_type == "directory" and not self._use_modern_installation:
+            return self._install_directory_without_wheel_installer(operation)
 
+        cleanup_archive: bool = False
         if package.source_type == "git":
-            return self._install_git(operation)
-
-        if package.source_type == "file":
-            archive = self._prepare_file(operation)
+            archive = self._prepare_git_archive(operation)
+            cleanup_archive = True
+        elif package.source_type == "file":
+            archive = self._prepare_archive(operation)
+        elif package.source_type == "directory":
+            archive = self._prepare_archive(operation)
+            cleanup_archive = True
         elif package.source_type == "url":
             assert package.source_url is not None
             archive = self._download_link(operation, Link(package.source_url))
@@ -485,14 +537,29 @@ class Executor:
             " <info>Installing...</info>"
         )
         self._write(operation, message)
-        return self.pip_install(archive, upgrade=operation.job_type == "update")
+
+        if not self._use_modern_installation:
+            return self.pip_install(archive, upgrade=operation.job_type == "update")
+
+        try:
+            if operation.job_type == "update":
+                # Uninstall first
+                # TODO: Make an uninstaller and find a way to rollback in case
+                # the new package can't be installed
+                assert isinstance(operation, Update)
+                self._remove(operation.initial_package)
+
+            self._wheel_installer.install(archive)
+        finally:
+            if cleanup_archive:
+                archive.unlink()
+
+        return 0
 
     def _update(self, operation: Install | Update) -> int:
         return self._install(operation)
 
-    def _remove(self, operation: Uninstall) -> int:
-        package = operation.package
-
+    def _remove(self, package: Package) -> int:
         # If we have a VCS package, remove its source directory
         if package.source_type == "git":
             src_dir = self._env.path / "src" / package.name
@@ -507,7 +574,7 @@ class Executor:
 
             raise
 
-    def _prepare_file(self, operation: Install | Update) -> Path:
+    def _prepare_archive(self, operation: Install | Update) -> Path:
         package = operation.package
         operation_message = self.get_operation_message(operation)
 
@@ -519,12 +586,48 @@ class Executor:
 
         assert package.source_url is not None
         archive = Path(package.source_url)
+        if package.source_subdirectory:
+            archive = archive / package.source_subdirectory
         if not Path(package.source_url).is_absolute() and package.root_dir:
             archive = package.root_dir / archive
 
+        self._populate_hashes_dict(archive, package)
+
+        return self._chef.prepare(archive, editable=package.develop)
+
+    def _prepare_git_archive(self, operation: Install | Update) -> Path:
+        from poetry.vcs.git import Git
+
+        package = operation.package
+        operation_message = self.get_operation_message(operation)
+
+        message = (
+            f"  <fg=blue;options=bold>•</> {operation_message}: <info>Cloning...</info>"
+        )
+        self._write(operation, message)
+
+        assert package.source_url is not None
+        source = Git.clone(
+            url=package.source_url,
+            source_root=self._env.path / "src",
+            revision=package.source_resolved_reference or package.source_reference,
+        )
+
+        # Now we just need to install from the source directory
+        original_url = package.source_url
+        package._source_url = str(source.path)
+
+        archive = self._prepare_archive(operation)
+
+        package._source_url = original_url
+
         return archive
 
-    def _install_directory(self, operation: Install | Update) -> int:
+    def _install_directory_without_wheel_installer(
+        self, operation: Install | Update
+    ) -> int:
+        from poetry.core.pyproject.toml import PyProjectTOML
+
         from poetry.factory import Factory
 
         package = operation.package
@@ -547,7 +650,12 @@ class Executor:
 
         pyproject = PyProjectTOML(os.path.join(req, "pyproject.toml"))
 
+        package_poetry = None
         if pyproject.is_poetry_project():
+            with contextlib.suppress(RuntimeError):
+                package_poetry = Factory().create_poetry(pyproject.file.path.parent)
+
+        if package_poetry is not None:
             # Even if there is a build system specified
             # some versions of pip (< 19.0.0) don't understand it
             # so we need to check the version of pip to know
@@ -557,69 +665,29 @@ class Executor:
                 < self._env.pip_version.__class__.from_parts(19, 0, 0)
             )
 
-            try:
-                package_poetry = Factory().create_poetry(pyproject.file.path.parent)
-            except RuntimeError:
-                package_poetry = None
+            builder: Builder
+            if package.develop and not package_poetry.package.build_script:
+                from poetry.masonry.builders.editable import EditableBuilder
 
-            if package_poetry is not None:
-                builder: Builder
-                if package.develop and not package_poetry.package.build_script:
-                    from poetry.masonry.builders.editable import EditableBuilder
+                # This is a Poetry package in editable mode
+                # we can use the EditableBuilder without going through pip
+                # to install it, unless it has a build script.
+                builder = EditableBuilder(package_poetry, self._env, NullIO())
+                builder.build()
 
-                    # This is a Poetry package in editable mode
-                    # we can use the EditableBuilder without going through pip
-                    # to install it, unless it has a build script.
-                    builder = EditableBuilder(package_poetry, self._env, NullIO())
-                    builder.build()
+                return 0
+            elif legacy_pip or package_poetry.package.build_script:
+                from poetry.core.masonry.builders.sdist import SdistBuilder
 
-                    return 0
-                elif legacy_pip or package_poetry.package.build_script:
-                    from poetry.core.masonry.builders.sdist import SdistBuilder
+                # We need to rely on creating a temporary setup.py
+                # file since the version of pip does not support
+                # build-systems
+                # We also need it for non-PEP-517 packages
+                builder = SdistBuilder(package_poetry)
+                with builder.setup_py():
+                    return self.pip_install(req, upgrade=True, editable=package.develop)
 
-                    # We need to rely on creating a temporary setup.py
-                    # file since the version of pip does not support
-                    # build-systems
-                    # We also need it for non-PEP-517 packages
-                    builder = SdistBuilder(package_poetry)
-
-                    with builder.setup_py():
-                        if package.develop:
-                            return self.pip_install(req, upgrade=True, editable=True)
-                        return self.pip_install(req, upgrade=True)
-
-        if package.develop:
-            return self.pip_install(req, upgrade=True, editable=True)
-
-        return self.pip_install(req, upgrade=True)
-
-    def _install_git(self, operation: Install | Update) -> int:
-        from poetry.vcs.git import Git
-
-        package = operation.package
-        operation_message = self.get_operation_message(operation)
-
-        message = (
-            f"  <fg=blue;options=bold>•</> {operation_message}: <info>Cloning...</info>"
-        )
-        self._write(operation, message)
-
-        assert package.source_url is not None
-        source = Git.clone(
-            url=package.source_url,
-            source_root=self._env.path / "src",
-            revision=package.source_resolved_reference or package.source_reference,
-        )
-
-        # Now we just need to install from the source directory
-        original_url = package.source_url
-        package._source_url = str(source.path)
-
-        status_code = self._install_directory(operation)
-
-        package._source_url = original_url
-
-        return status_code
+        return self.pip_install(req, upgrade=True, editable=package.develop)
 
     def _download(self, operation: Install | Update) -> Path:
         link = self._chooser.choose_for(operation.package)
@@ -641,11 +709,13 @@ class Executor:
     def _download_link(self, operation: Install | Update, link: Link) -> Path:
         package = operation.package
 
-        archive = self._chef.get_cached_archive_for_link(link)
-        if archive is None:
-            # No cached distributions was found, so we download and prepare it
+        output_dir = self._chef.get_cache_directory_for_link(link)
+        # Try to get cached original package for the link provided
+        original_archive = self._chef.get_cached_archive_for_link(link, strict=True)
+        if original_archive is None:
+            # No cached original distributions was found, so we download and prepare it
             try:
-                archive = self._download_archive(operation, link)
+                original_archive = self._download_archive(operation, link)
             except BaseException:
                 cache_directory = self._chef.get_cache_directory_for_link(link)
                 cached_file = cache_directory.joinpath(link.filename)
@@ -656,18 +726,36 @@ class Executor:
 
                 raise
 
-        if package.files:
-            archive_hash = self._validate_archive_hash(archive, package)
+        # Get potential higher prioritized cached archive, otherwise it will fall back
+        # to the original archive.
+        archive = self._chef.get_cached_archive_for_link(link, strict=False)
+        # 'archive' can at this point never be None. Since we previously downloaded
+        # an archive, we now should have something cached that we can use here
+        assert archive is not None
 
-            self._hashes[package.name] = archive_hash
+        if archive.suffix != ".whl":
+            message = (
+                f"  <fg=blue;options=bold>•</> {self.get_operation_message(operation)}:"
+                " <info>Preparing...</info>"
+            )
+            self._write(operation, message)
+
+            archive = self._chef.prepare(archive, output_dir=output_dir)
+
+        # Use the original archive to provide the correct hash.
+        self._populate_hashes_dict(original_archive, package)
 
         return archive
 
+    def _populate_hashes_dict(self, archive: Path, package: Package) -> None:
+        if package.files and archive.name in {f["file"] for f in package.files}:
+            archive_hash = self._validate_archive_hash(archive, package)
+            self._hashes[package.name] = archive_hash
+
     @staticmethod
     def _validate_archive_hash(archive: Path, package: Package) -> str:
-        file_dep = FileDependency(package.name, archive)
-        archive_hash: str = "sha256:" + file_dep.hash()
-        known_hashes = {f["hash"] for f in package.files}
+        archive_hash: str = "sha256:" + get_file_hash(archive)
+        known_hashes = {f["hash"] for f in package.files if f["file"] == archive.name}
 
         if archive_hash not in known_hashes:
             raise RuntimeError(
@@ -706,7 +794,7 @@ class Executor:
         done = 0
         archive = self._chef.get_cache_directory_for_link(link) / link.filename
         archive.parent.mkdir(parents=True, exist_ok=True)
-        with archive.open("wb") as f:
+        with atomic_open(archive) as f:
             for chunk in response.iter_content(chunk_size=4096):
                 if not chunk:
                     break
@@ -726,7 +814,9 @@ class Executor:
         return archive
 
     def _should_write_operation(self, operation: Operation) -> bool:
-        return not operation.skipped or self._dry_run or self._verbose
+        return (
+            not operation.skipped or self._dry_run or self._verbose or not self._enabled
+        )
 
     def _save_url_reference(self, operation: Operation) -> None:
         """
@@ -768,7 +858,8 @@ class Executor:
             for dist in self._env.site_packages.distributions(
                 name=package.name, writable_only=True
             ):
-                dist_path = cast(Path, dist._path)  # type: ignore[attr-defined]
+                dist_path = dist._path  # type: ignore[attr-defined]
+                assert isinstance(dist_path, Path)
                 url = dist_path / "direct_url.json"
                 url.write_text(json.dumps(url_reference), encoding="utf-8")
 
@@ -794,20 +885,12 @@ class Executor:
         return reference
 
     def _create_url_url_reference(self, package: Package) -> dict[str, Any]:
-        archive_info = {}
+        archive_info = self._get_archive_info(package)
 
-        if package.name in self._hashes:
-            archive_info["hash"] = self._hashes[package.name]
-
-        reference = {"url": package.source_url, "archive_info": archive_info}
-
-        return reference
+        return {"url": package.source_url, "archive_info": archive_info}
 
     def _create_file_url_reference(self, package: Package) -> dict[str, Any]:
-        archive_info = {}
-
-        if package.name in self._hashes:
-            archive_info["hash"] = self._hashes[package.name]
+        archive_info = self._get_archive_info(package)
 
         assert package.source_url is not None
         return {
@@ -826,3 +909,20 @@ class Executor:
             "url": Path(package.source_url).as_uri(),
             "dir_info": dir_info,
         }
+
+    def _get_archive_info(self, package: Package) -> dict[str, Any]:
+        """
+        Create dictionary `archive_info` for file `direct_url.json`.
+
+        Specification: https://packaging.python.org/en/latest/specifications/direct-url
+        (it supersedes PEP 610)
+
+        :param package: This must be a poetry package instance.
+        """
+        archive_info = {}
+
+        if package.name in self._hashes:
+            algorithm, value = self._hashes[package.name].split(":")
+            archive_info["hashes"] = {algorithm: value}
+
+        return archive_info
